#!/bin/bash
PROGRAM="$1"
STARTDATE="$(date --iso=s)"
# temporary directory - must be empty
mkdir "${PROGRAM}"
cd "${PROGRAM}"
# generate frames
../ulci "../${PROGRAM}.ulc"
# duplicate last frame to get multiple of 15 frames
COUNT="$(ls -1 | wc -l)"
C="${COUNT}"
for (( ; C % 15 ; ++C ))
do
  cp "${COUNT}.svg" "$((C+1)).svg"
done
# duplicate last frame 75 more times
COUNT="$(ls -1 | wc -l)"
C="${COUNT}"
for (( ; C < COUNT + 75 ; ++C ))
do
  cp "${COUNT}.svg" "$((C+1)).svg"
done
# shift frames up by 75 to make room
COUNT="$(ls -1 | wc -l)"
C="${COUNT}"
for (( ; C ; --C ))
do
  mv "${C}.svg" "$((C+75)).svg"
done
# duplicate first frame 75 more times
C="75"
for (( ; C ; --C ))
do
  cp "76.svg" "${C}.svg"
done
# pad names with 0 so "*.svg" lists in numerical order
rename "s/^/0/" ?.svg
rename "s/^/0/" ??.svg
rename "s/^/0/" ???.svg
# generate score
for f in *.svg
do
  r="$(cat "${f}" | tr '>' '\n' | grep "'red'"    | wc -l)"
  y="$(cat "${f}" | tr '>' '\n' | grep "'yellow'" | wc -l)"
  g="$(cat "${f}" | tr '>' '\n' | grep "'green'"  | wc -l)"
  c="$(cat "${f}" | tr '>' '\n' | grep "'cyan'"   | wc -l)"
  b="$(cat "${f}" | tr '>' '\n' | grep "'blue'"   | wc -l)"
  echo "${r} ${y} ${g} ${c} ${b} ;"
done > "../${PROGRAM}.score"
# generate audio
pd -nogui -stderr -verbose -nrt -noaudio -r 48000 -open ../sonify.pd -send \
  "; GO ${PROGRAM}.score ${PROGRAM}_a.wav"
# encode audio to Ogg Vorbis
oggenc -b 192 -o "../${PROGRAM}_a.ogg" "../${PROGRAM}_a.wav"
# encode audio to DVD MP2
twolame -b 224 "../${PROGRAM}_a.wav" "../${PROGRAM}_a.mp2"
# convert to PNG
for f in *.svg
do
  rsvg "${f}" "${f}.png"
done
# encode video to HD Ogg Theora
ffmpeg2theora -f image2 %04d.svg.png  -o "../${PROGRAM}_v_hd.ogg"
# encode video to LD Ogg Theora
ffmpeg2theora -p videobin -f image2 %04d.svg.png  -o "../${PROGRAM}_v_ld.ogg"
# encode video to DVD MPEG2
ffmpeg -f image2 -i %04d.svg.png -pix_fmt yuv444p -f yuv4mpegpipe - |
y4mscaler -I sar=1/1 -I bg=RGB:255,255,255 \
  -O preset=dvd_wide -O bg=RGB:255,255,255 |
mpeg2enc -f 8 -q 2 -b 8000 -o "../${PROGRAM}_v.m2v"
# multiplex videos
oggzmerge -o "../${PROGRAM}_hd.ogg" "../${PROGRAM}_v_hd.ogg" "../${PROGRAM}_a.ogg"
oggzmerge -o "../${PROGRAM}_ld.ogg" "../${PROGRAM}_v_ld.ogg" "../${PROGRAM}_a.ogg"
mplex -f 8 -V -o "../${PROGRAM}_dvd.mpeg" "../${PROGRAM}_v.m2v" "../${PROGRAM}_a.mp2"
# cleanup
cd ".."
mv -t "${PROGRAM}" \
  "${PROGRAM}.score" "${PROGRAM}_a.wav" "${PROGRAM}_a.ogg" "${PROGRAM}_a.mp2" \
  "${PROGRAM}_v_hd.ogg" "${PROGRAM}_v_ld.ogg" "${PROGRAM}_v.m2v"
# stats
echo "initiated: ${STARTDATE}"
du -hs "${PROGRAM}/" "${PROGRAM}_hd.ogg" "${PROGRAM}_ld.ogg" "${PROGRAM}_dvd.mpeg"
echo "completed: $(date --iso=s)"
